﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Interop;
using System.Windows.Media;

namespace Simano.Service.Adapter
{
    public unsafe class CubeProgrammerAPIAdapter
    {
        [DllImport("CubeProgrammer_API.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern int setLoadersPath([MarshalAs(UnmanagedType.LPWStr)] string path);

        [DllImport("CubeProgrammer_API.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern int setDisplayCallbacks(displayCallBacks usartList);

        [DllImport("CubeProgrammer_API.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern int getUsartList(usartConnectParameters** usartList);

        [DllImport("CubeProgrammer_API.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern int connectUsartBootloader(usartConnectParameters param);

        [DllImport("CubeProgrammer_API.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern int downloadFile([MarshalAs(UnmanagedType.LPWStr)] string filePath, UInt32 address, UInt32 skipErase, UInt32 verify, [MarshalAs(UnmanagedType.LPWStr)] string binPath);

        [DllImport("CubeProgrammer_API.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern void disconnect();

        [DllImport("CubeProgrammer_API.dll", CallingConvention = CallingConvention.StdCall, CharSet = CharSet.Ansi)]
        public static extern int execute(UInt32 address);

        public static event EventHandler<string> OnLogReceived;

        private static displayCallBacks callBack;

        public static void Init()
        {
            setLoadersPath("./.");
            callBack = default;
            callBack.initProgressBar = InitP;
            callBack.logMessage = logMessageDelegate;
            callBack.loadBar = loadBar;
            setDisplayCallbacks(callBack);
        }

        public static List<string> GetUsartList()
        {
            var usartList = GetRawUsartList();
            return usartList.Select(uartParam =>
            {
                byte[] arr = new byte[100];
                Marshal.Copy((IntPtr)uartParam.portName, arr, 0, 100);
                var portName = Encoding.ASCII.GetString(arr).Trim('\0');
                return portName;
            }).ToList();
        }

        public static int openPort(string name, int? baudrate = null)
        {
            var usartList = GetRawUsartList();
            usartConnectParameters uart = default;
            bool foundUart = false;
            foreach (var uartParam in usartList)
            {
                byte[] arr = new byte[100];
                Marshal.Copy((IntPtr)uartParam.portName, arr, 0, 100);
                var portName = Encoding.ASCII.GetString(arr).Trim('\0');
                if (portName == name)
                {
                    foundUart = true;
                    uart = uartParam;
                    break;
                }
            }
            if (!foundUart)
            {
                return -123;
            }
            if (baudrate != null)
            {
                uart.baudrate = (uint)baudrate;
            }
            var openResult = connectUsartBootloader(uart);
            return openResult;
        }

        public static void ClosePort()
        {
            disconnect();
        }

        public static bool Run(uint address)
        {
            var result =  execute(address);
            return result == 0;
        }

        private static List<usartConnectParameters> GetRawUsartList()
        {
            usartConnectParameters* uartParam;
            int getUsartListFlag = getUsartList(&uartParam);
            var toReturn = new List<usartConnectParameters>();

            if (getUsartListFlag == 0)
            {
                return toReturn;
            }

            for (int i = 0; i < getUsartListFlag; i++)
            {
                toReturn.Add(uartParam[i]);
            }
            return toReturn;
        }

        private static void InitP()
        {

        }

        private static void logMessageDelegate(int msgType, string strp)
        {
            //var msg = Encoding.ASCII.GetString(str).Trim('\0');
            //OnLogReceived?.Invoke(null, msg);
            //OnLogReceived?.Invoke(null, str);
            OnLogReceived?.Invoke(null, strp);

            //byte[] arr = new byte[100];
            //Marshal.Copy(strp, arr, 0, 100);
            //var msg = Encoding.ASCII.GetString(arr).Trim('\0');
            //OnLogReceived?.Invoke(null, msg);

            //var msg = Encoding.ASCII.GetString(strp).Trim('\0');
            //OnLogReceived?.Invoke(null, msg);
        }

        private static void loadBar(int x, int n)
        {

        }
    }
}
