﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Security.Claims;
using System.Text;
using System.Threading.Tasks;

namespace Simano.Service.Adapter
{
    [StructLayout(LayoutKind.Sequential)]
    public unsafe struct usartConnectParameters
    {
        //[MarshalAs(UnmanagedType.ByValArray, SizeConst = 100, ArraySubType = UnmanagedType.LPUTF8Str)]
        public fixed Byte portName[100];             /**< Interface identifier: COM1, COM2, /dev/ttyS0...*/
        public UInt32 baudrate;         /**< Speed transmission: 115200, 9600... */
        public usartParity parity;             /**< Parity bit: value in usartParity. */
        public Byte dataBits;         /**< Data bit: value in {6, 7, 8}. */
        public Single stopBits;                 /**< Stop bit: value in {1, 1.5, 2}. */
        public usartFlowControl flowControl;   /**< Flow control: value in usartFlowControl. */
        public Int32 statusRTS;                  /**< RTS: Value in {0,1}. */
        public Int32 statusDTR;                  /**< DTR: Value in {0,1}. */
        public Byte noinitBits;       /**< Set No Init bits: value in {0,1}. */
        public Byte rdu;                       /**< request a read unprotect: value in {0,1}.*/
        public Byte tzenreg;                       /**< request a TZEN regression: value in {0,1}.*/
    }

    public struct displayCallBacks
    {
        public initProgressBarDelegate initProgressBar;
        public logMessageDelegate logMessage;
        public loadBarDelegate loadBar;
    }

    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void initProgressBarDelegate();
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void logMessageDelegate(int msgType, [MarshalAs(UnmanagedType.LPWStr)] string strp);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void loadBarDelegate(int x, int n);

    public enum usartParity : UInt32
    {
        EVEN = 0,               /**< Even parity bit. */
        ODD = 1,                /**< Odd parity bit. */
        NONE = 2,               /**< No check parity. */
    };

    public enum usartFlowControl : UInt32
    {
        OFF = 0,                /**< No flow control.  */
        HARDWARE = 1,           /**< Hardware flow control : RTS/CTS.  */
        SOFTWARE = 2,           /**< Software flow control : Transmission is started and stopped by sending special characters. */
    }

    public enum debugResetMode
    {
        SOFTWARE_RESET = 0,         /**< Apply a reset by the software. */
        HARDWARE_RESET = 1,         /**< Apply a reset by the hardware. */
        CORE_RESET = 2              /**< Apply a reset by the internal core peripheral. */
    };
}
