﻿using Simano.Service.Adapter;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Simano.Service.CubeProgrammerAPI
{
    public static class CubeProgrammerAPIWrapper
    {
        private delegate int downloadFileCaller(string filePath, UInt32 address, UInt32 skipErase, UInt32 verify, string binPath);
        private delegate int executeCaller(UInt32 address);
        private delegate int openPortCaller(string name, int? baudrate = null);

        private static SemaphoreSlim semaphoreSlim = new SemaphoreSlim(0);

        public static async Task<bool> OpenPort(string name, int? baudrate = null)
        {
            var method = new openPortCaller(CubeProgrammerAPIAdapter.openPort);
            IAsyncResult result = method.BeginInvoke(name, baudrate, ActionCompleted, null);
            await semaphoreSlim.WaitAsync();
            int openResult = method.EndInvoke(result);
            return openResult == 0;
        }

        public static async Task<bool> DownloadFile(string flashFilePath, uint address)
        {
            var method = new downloadFileCaller(CubeProgrammerAPIAdapter.downloadFile);
            IAsyncResult result = method.BeginInvoke(flashFilePath, address, 0, 1, "", ActionCompleted, null);
            await semaphoreSlim.WaitAsync();
            int downloadResult = method.EndInvoke(result);
            return downloadResult == 0;
        }

        public static async Task<bool> Run(uint address)
        {
            var method = new executeCaller(CubeProgrammerAPIAdapter.execute);
            IAsyncResult result = method.BeginInvoke(address, ActionCompleted, null);
            await semaphoreSlim.WaitAsync();
            int runResult = method.EndInvoke(result);
            return runResult == 0;
        }

        private static void ActionCompleted(IAsyncResult ar)
        {
            semaphoreSlim.Release();
        }
    }
}
